;----------------------------------------------------------------------------
;
; Min2Tray
;
; _v1.7.0, 2006/08/25
;
; _minimize a window to system tray area of taskbar as icon,
;  make window always-on-top or maximize window vertically.
;  requires ms windows nt/2000/xp or newer.
;
; _created by Junyx / KTC^brain in June 2005
;
; _licensed under the terms of the GPLv2
;
; _program is a 2-in-1 thingy:
;   _started without parameters on cmdline it behaves as
;    the main program waiting for hotkey action
;   _startet with ahk_window_id on cmdline by main program
;    it will hide that particular window and generate
;    a (hopefully) meaningful tray icon
;
; _inspired by:
;   ActualWindowMinimizer (unfortunately payware)
;   NiftyWindows by Enovatic-Solutions (nice tweaks)
;   ac'tivAid by Heise Zeitschriften Verlag GmbH & Co. KG
;
; _contains some code from:
;   minimize.ahk (from AHK docs, no author)
;   Icon Viewer Lite v1 by toralf
;
; _thanks to:
;   Chris Mallett (support response in a flash!)
;   Rajat (for some wonderful scripts)
;   GSX-R600 / KTC^game (for testing)
;   Lemmy / KTC^game (for testing)
;   all users who gave feedback
;
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
; init
;
#NoEnv
#MaxMem 1
#Persistent
#NoTrayIcon
#KeyHistory 0
#SingleInstance Off

versionString = 1.7.0
versionDate   = 20060825

; language, check-ups etc.
Gosub, h_InitGeneral

#UseHook On
#WinActivateForce
StringCaseSense, On
DetectHiddenWindows, On
SetWinDelay 10

If ( %0% < 1 )	; cmdline: no args -> behaviour: starter
{
   ;=========================================================================
   ; started as hotkey-launcher (starter)
	Goto, h_InitStarter
	Return
}
; end of auto-execution for starter here!
;============================================================================

;============================================================================
; function as tray menu for minimized window (helper) starts here

;----------------------------------------------------------------------------
; Helper -- init
;
If ( h_Language = "de_DE" )
{
   lng_MenuNameChange         = &Eigenen Name zuweisen...
   lng_MenuNameRemove         = &Entferne eigenen Name
   lng_MenuIconChange         = &Eigenes Icon zuweisen...
   lng_MenuIconRemove         = &Entferne eigenes Icon
   lng_MenuMultiWin           = &Liste der minimierten Fenster
   lng_MenuAssignHotkey       = &Hotkey zum Fenster-Anzeigen...
   lng_MenuClose              = &Schliee Fenster
   lng_MenuUnhide             = &Zeige Fenster
   lng_MenuOnBKList           = &Fenster ist auf BossKey-Liste
   lng_MenuShowOnTitleChange  = &Zeige Fenster bei gendertem Titel
   lng_hClose1                = Wollen Sie das Fenster "
   lng_hClose2                = " wirklich schlieen?
   lng_ChangeNameSelector     = Geben Sie bitte den neuen Namen ein:
   lng_ChangeIconSelector     = Whlen Sie eine Icon-Datei aus fr "
   lng_ChangeIconSelectorExt  = Icon-Dateien
   lng_ChangeIconMulti        = Whlen Sie ein Icon aus fr "
   lng_RemoveName             = Wollen Sie den benutzerdefinierten Name wirklich entfernen?
   lng_RemoveIcon             = Wollen Sie das benutzerdefinierte Icon wirklich entfernen?
   lng_SetupUnhideKey         = Hotkey zum Fenster zeigen
   ; Errors
   lng_ExitWithParamError     = Fehler: Keine Nummer als erster Parameter bergeben!
}
Else
{
   lng_MenuNameChange         = &Assign custom name...
   lng_MenuNameRemove         = &Remove custom name
   lng_MenuIconChange         = &Assign custom icon...
   lng_MenuIconRemove         = &Remove custom icon
   lng_MenuMultiWin           = &List of minimized windows
   lng_MenuAssignHotkey       = &Hotkey for showing window(s)...
   lng_MenuClose              = &Close window(s)
   lng_MenuUnhide             = &Show window(s)
   lng_MenuOnBKList           = &Window is on BossKey list
   lng_MenuShowOnTitleChange  = &Show window when title changes
   lng_hClose1                = Do you really wanna close window(s) "
   lng_hClose2                = "?
   lng_ChangeNameSelector     = Please enter the custom name:
   lng_ChangeIconSelector     = Select an icon file for "
   lng_ChangeIconSelectorExt  = Icon files
   lng_ChangeIconMulti        = Select an icon for "
   lng_RemoveName             = Do you really wanna remove the custom name?
   lng_RemoveIcon             = Do you really wanna remove the custom icon?
   lng_SetupUnhideKey         = Hotkey to show window(s)
   ; Errors
   lng_ExitWithParamError     = Error: First parameter must be a number!
}

h_ID0 = 0
Loop, %0%   ; go thru all arguments on command line
{
   tmp := %A_Index%

   If InStr( tmp, "0x", TRUE ) = 1
   {  ; get several (at least one) window id
      h_ID0++
      h_ID%h_ID0% = %tmp%
   }
   Else If ( h_ID0 = 0 )   ; get title for multi window mode
      h_MultiTitle = %tmp%
}

If ( h_ID0 = 0 )
   Goto, h_ExitWithParamError

; h_ID0 now holds the count of window IDs used (1 or more)
; h_ID1 is the first window ID, h_ID2 the next and so on

newID0 = 0  ; count of really hidden windows
Loop, %h_ID0%
{
   h_WinID := h_ID%A_Index%
   Loop
   {
      skipThis := FALSE

      ; set "last used window"
      WinWait, ahk_id %h_WinID%,, 1
      If ( ErrorLevel ) ; It timed out, so go on with next window id
      {
         skipThis := TRUE   ; outer loop skip
      	Break
      }
      
      WinGetClass, h_Class
      WinGet, h_Application, ProcessName
   
      ; class substitution (needed only for evil classes :-)
   	RegRead, goodClass, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ec	; look for evil class
      If ( Not ErrorLevel )
      {  ; get new h_WinID (unique window id) with help of good class and old PID
         WinGet, h_PID, PID
         WinGet, h_WinID, ID, ahk_pid %h_PID% ahk_class %goodClass%
         ; go another looping round with new content in h_WinID...
      }
      Else
         Break ; leave the loop!
   }
   If ( skipThis )
      Continue ; skipping since window couldn't be made "last used one"

   If h_Class in Shell_TrayWnd,Progman,WorkerW
   	Continue

   WinGetTitle, h_Title
   WinGet, h_MinMax, MinMax

   If ( h_MinMax <> -1 )
   {  ; only minimize window to taskbar prior to hiding if not already done
      WinMinimize
      Sleep, 75
   }
   WinHide
   newID0++ ; one more win hidden
   newID%newID0% = %h_WinID%  ; update with substituted or original class
   newApp%newID0% = %h_Application%
   newTit%newID0% = %h_Title%
}

If ( newID0 = 0 )
   ExitApp, 1  ; no windows found for hiding

; newID0 now holds the count of window IDs hidden (1 or more)
; newID1 is the first window ID, newID2 the next and so on
; newApp1 is the first application name
; newTit1 is the first window title
; at least newID1 __must__ be assigned!

; re-activate last used window after hiding 
Sleep, 50
;SendInput, !{ESC}   ; old-style way
WinActivateBottom, A

If ( newID0 > 1 )
{  ; set new vars when in multi mode
   h_Application = __multi__
   h_Title = %h_MultiTitle%
   StringReplace, tmp, h_MultiTitle, %A_SPACE%, , All
   StringReplace, tmp, tmp, %A_TAB%, , All
   StringReplace, tmp, tmp, `r`n, , All
   h_Class = %tmp%
}

OnExit, h_Unhide
Gosub, h_SetTrayAndMenu
Gosub, h_SetTimedCheck

; assign user-hotkey, if any
h_hkKey =
RegRead, h_hkKey, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ch
f_AssignHotkey( h_hkKey, "h_Unhide" )

Return	; end of auto execution, wait for menu action

;----------------------------------------------------------------------------
; Helper -- subs
;

h_Unhide:
   OnExit
   SetTimer, h_CheckForWin, Off
   ; flushes the hotkey buffer before exiting
   ; no more detection of hotkey modifier WIN under windows desktop
   Suspend, On
   Loop, %newID0%
   {  ; open windows in reverse order
      tmp := newID0 - (A_Index - 1)
      h_WinID := newID%tmp%
      If (h_WinID = "")
         Continue
      WinShow, ahk_id %h_WinID%
      WinRestore, ahk_id %h_WinID%
      WinActivate, ahk_id %h_WinID%
   }
ExitApp

h_Close:
   Msgbox, 36, %h_MenuWin% - %lng_WindowTitle%, %lng_hClose1%%h_MenuWin%%lng_hClose2%
   IfMsgBox, No	; do not close
      Return

   OnExit
   SetTimer, h_CheckForWin, Off
   Suspend, On
   Loop, %newID0%
   {
      h_WinID := newID%A_Index%
      If (h_WinID = "")
         Continue
      WinShow, ahk_id %h_WinID%
      WinClose, ahk_id %h_WinID%
   }
ExitApp

h_SetTimedCheck:
   ; get interval from registry, if any
	RegRead, tmp, HKCU, %h_RegSubkey%\Misc, CheckForWinEveryMiliSec
	If ( ErrorLevel OR tmp < 200 OR tmp > 1000000 )
		tmp = 2500
	SetTimer, h_CheckForWin, %tmp%
Return

h_CheckForWin:
   hiddenWin = %newID0%
   Loop, %newID0%
   {
      h_WinID := newID%A_Index%
      ; window already unhidden or non-existing?
      WinGet, tmp, Style, ahk_id %h_WinID%
      Transform, tmp, BitAnd, %tmp%, 0xF0000000
      If ( tmp = 0x10000000 OR Not WinExist( "ahk_id " h_WinID ) )
      {  ; decrement count of hidden windows
         hiddenWin--
         ; MultiMode: remove entry from sub menu if win ID is still valid
			If ( (newID0 > 1) AND (h_WinID <> "") )
			{
	         tmpOld := f_StringLeft( A_Index ": " newTit%A_Index% " [" newApp%A_Index% "]" )
	        	Menu, MultiWinMenu, Delete, %tmpOld%
	         newID%A_Index% = ; invalidate win ID
			}
      }
		Else
		{
		   ; get current title
			WinGetTitle, tmpNew, ahk_id %h_WinID%		
         ; check for changed window title
		   If ( newID0 = 1 )
		   {  ; NOT in MultiMode
		      If ( tmpNew <> h_Title )
		      {
		         If ( h_sotcEnabled ) ; show window after title changed
		            Gosub, h_Unhide
		         Else
		         {  ; update window title and redraw menu
		            h_Title = %tmpNew%
		            Gosub, h_SetMenu
		         }
		      }
		   }
		   Else
		   {  ; we are in MultiMode
	         If ( tmpNew <> newTit%A_Index% )
	         {
	            tmpOld := f_StringLeft( A_Index ": " newTit%A_Index% " [" newApp%A_Index% "]" )
	            newTit%A_Index% = %tmpNew%
	            tmp    := f_StringLeft( A_Index ": " newTit%A_Index% " [" newApp%A_Index% "]" )
	           	Menu, MultiWinMenu, Rename, %tmpOld%, %tmp%
	         }
		   }
		}
   }
   If ( hiddenWin <= 0 )
   {  ; there are no hidden windows anymore
      OnExit
      ExitApp
   }

   ; redraw tray, if changed
   f_ShowTrayIcon( h_appExe, h_appExeIcon )

   ; inspect regitry for changed BossKey entry for our window
   RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|bk
   If ( ErrorLevel )
      tmp := FALSE
   If ( ( tmp AND !h_bkEnabled ) OR ( !tmp AND h_bkEnabled ) )
      Gosub, h_BossKeyListToggle ; set menu text new
Return

h_ChangeName:
	SysGet, tmp, 15	; caption height - for rough computation of dialog height
	tmp *= 7	; old hardcoded default height was: 125
   InputBox, h_appName, %h_MenuWin% - %lng_WindowTitle%, %lng_ChangeNameSelector%, , , %tmp%, , , , , %h_MenuWin%
   If ( Not ErrorLevel )
   {
      StringLen, tmp, h_appName
      If ( tmp < 1 )
         Return
      
      RegWrite, REG_SZ, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|cn, %h_appName%

      If ( Not m2t_JaakonMode )
         Gosub, h_SetMenu
   }
   ; ... returns with ErrorLevel still set - needed by h_SetMenu
Return

h_RemoveName:
   RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|cn	; does custom name exist?
   If ( Not ErrorLevel )
   {
      Msgbox, 36, %h_MenuWin% - %lng_WindowTitle%, %lng_RemoveName%
      IfMsgBox, No	; do not remove
         Return

      RegDelete, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|cn
      If ( Not ErrorLevel )
      {
         m2t_JaakonMode := FALSE   ; disable Jaakon's Mode to set script built name
         Gosub, h_SetMenu
      }
   }
Return

h_ChangeIcon:
   Suspend, On
	newIconFile =
   hCIwinTitle = %lng_ChangeIconSelector%%h_MenuWin%" - %lng_WindowTitle%
	hCIcount = 1000
   
   SetTimer, h_ChangeIcon_MoveWin, 1   ; center file selector window upon opening
	FileSelectFile, newIconFile, 1, %h_appExe% , %hCIwinTitle%, %lng_ChangeIconSelectorExt% (*.ico; *.cur; *.ani; *.exe; *.dll; *.cpl; *.scr; *.drv; *.ocx; *.vbx; *.acm; *.bpl)
	If FileExist( newIconFile )
	{
	   FileGetAttrib, tmp, %newIconFile%
	   If InStr( tmp, "D" )   ; no folders!
	      Goto, 3SuspendOff   ; outta here

      Gui, 3:Default
      Gui, +AlwaysOnTop -MaximizeBox -MinimizeBox
      Gui, Add, ListView, Section r6 w480 -Multi AltSubmit +Icon gLVevent, IconNumber
      ;Gui, Add, Button, h0 w0 Hidden g3BtnOK, OK
      Gui, Add, Button, ys Section w90 Disabled g3BtnOK, %lng_SetupOK%
      Gui, Add, Button, xs wp g3GuiClose, %lng_SetupCancel%

      ;clear list of files 
      LV_Delete() 
      ;list of images 
      imageList := IL_Create(100, 100, 2)
      iLID := LV_SetImageList(imageList)
      If ( iLID ) 
         IL_Destroy(iLID) 

      icons = 0
      Loop, 9999 
      { 
         iconIndex := IL_Add(imageList, newIconFile, A_Index) 
         If ( iconIndex > 0 )
         { 
            icons++ 
            LV_Add("Icon" . A_Index, icons)
         } 
         Else 
            Break 
      } 

      userIcon = 0   ; no icon as default
      If ( icons = 1 )
      {
         userIcon = 1
         Goto, 3SetIconAndOut
      }
      Else If ( icons = 0 )   ; no icons in file!
         Goto, 3SuspendOff   ; outta here
          
   	hCIguiUsed := TRUE
      noDefaultBtn := TRUE
      Gui, Show, Center, %lng_ChangeIconMulti%%h_MenuWin%" - %lng_WindowTitle%
      Goto, 3SuspendOff   ; outta here
      
      LVevent:
         If ( A_GuiControlEvent = "I" )
         {
            If ( noDefaultBtn )  ; no icon selected, yet
            {
               GuiControl, Enabled, %lng_SetupOK%
               GuiControl, +Default, %lng_SetupOK%
               noDefaultBtn := FALSE
            }

            rowNumber := LV_GetNext() 
            If ( rowNumber ) 
               LV_GetText(userIcon, rowNumber, 1)
            Else
            {
               userIcon = 0 
               GuiControl, Disable, %lng_SetupOK%
               noDefaultBtn := TRUE
            }
         }

         If ( A_GuiControlEvent = "DoubleClick" )
         {
            If ( noDefaultBtn )  ; no icon selected, yet
               Return

            rowNumber := LV_GetNext() 
            If ( rowNumber ) 
               LV_GetText(userIcon, rowNumber, 1)
            Else
               userIcon = 0 
            
            Gosub, 3BtnOK
         }
      Return

      3BtnOK:
         If ( userIcon = 0 )
            Return
      3SetIconAndOut:
         h_appExe     := newIconFile
         h_appExeIcon := userIcon
         RegWrite, REG_SZ, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ci, %h_appExe%|%h_appExeIcon%
         Gosub, h_SetTrayAndMenu
      3GuiClose:
      3GuiEscape:
         Gui, Destroy
         hCIguiUsed := FALSE
      3SuspendOff:
         Suspend, Off
      Return   ; mandatory due to nesting!
	}
	Suspend, Off
	Return

   h_ChangeIcon_MoveWin:
      IfWinActive, %hCIwinTitle%
      {
         WinGetPos,,, hCIWidth, hCIHeight
      	WinMove,,, (A_ScreenWidth/2)-(hCIWidth/2), (A_ScreenHeight/2)-(hCIHeight/2)
      	SetTimer, h_ChangeIcon_MoveWin, Off
      }
      Else
      {
         hCIcount--
         If ( hCIcount = 0 )  ; switch timer always off after "hCIcount" rounds
            SetTimer, h_ChangeIcon_MoveWin, Off
      }
   Return
Return

h_RemoveIcon:
   RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ci	; does custom icon exist?
   If ( Not ErrorLevel )
   {
      Msgbox, 36, %h_MenuWin% - %lng_WindowTitle%, %lng_RemoveIcon%
      IfMsgBox, No	; do not remove
         Return

      RegDelete, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ci
      If ( Not ErrorLevel )
         Gosub, h_SetTrayAndMenu
   }
Return

h_SetTrayAndMenu:
   h_ciEnabled := FALSE

	RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|ci	; first look for user icon
	If ( Not ErrorLevel )
	{
		StringSplit, tmparray, tmp, |
		h_appExe     = %tmparray1%
		h_appExeIcon = %tmparray2%
		h_ciEnabled  := TRUE
	}
	Else
	{
		RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|ci	; look for generic user icon (for app)
		If ( Not ErrorLevel )
		{
			StringSplit, tmparray, tmp, |
			h_appExe     = %tmparray1%
			h_appExeIcon = %tmparray2%
   		h_ciEnabled  := TRUE
		}
		Else If ( ( h_Application = "explorer.exe" ) AND ( h_Class = "ExploreWClass" ) )
		{	; special treatment for explorer windows
			h_appExe     = %A_WinDir%\system32\shell32.dll
			h_appExeIcon = 5
		}
		Else
		{	; use icon from registry-entry as default, if existing
			h_appExeIcon = 1
			RegRead, h_appExe, HKLM, SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\%h_Application%
			If ( h_appExe = "" )	; otherwise do a file search
			{
				h_appExe := f_FindFile( h_Application, A_WinDir )
				If ( h_appExe = "" )	; nothing found in windir...
					h_appExe := f_FindFile( h_Application, A_ProgramFiles )
			}
		}
	}
   ; update the menu entries and show icon
	Gosub, h_SetMenu
	f_ShowTrayIcon( h_appExe, h_appExeIcon )
Return

h_SetMenu:
   h_cnEnabled := FALSE ; custom name in use?

	RegRead, h_appName, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|cn 	; look for custom name
	If ( Not ErrorLevel )
	{
		h_MenuWin   = %h_appName%
		h_TrayTitle = %h_appName%
		h_cnEnabled := TRUE
	}
	Else
	{
		RegRead, h_appName, HKCU, %h_RegSubkey%, %h_Application%|cn 	; look for generic custom name
		If ( Not ErrorLevel )
		{
			h_MenuWin   = %h_appName%
			h_TrayTitle = %h_appName%
			h_cnEnabled := TRUE
		}
		Else
		{
         h_Title    := f_StringLeft( h_Title )
			h_MenuWin   = %h_Title% [%h_Application%]
			h_TrayTitle = %h_Title%`n[%h_Application%]

         ; Jaakon's Mode asks for custom name if none is specified
         If ( m2t_JaakonMode )
         {
   		   Gosub, h_ChangeName
   		   If ( Not ErrorLevel ) ; only set vars if OK was pressed in input box within h_ChangeName 
   		   {
      			h_MenuWin   = %h_appName%
      			h_TrayTitle = %h_appName%
      			h_cnEnabled := TRUE
   		   }
   		   m2t_JaakonMode := FALSE
   		}
		}
	}

	; is this window on BossKey list?
	RegRead, h_bkEnabled, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|bk	; app+class on bosskey list?
   If ( ErrorLevel )
      h_bkEnabled := FALSE	; default: nope

   ; ShowOnTitleChange enabled?
   RegRead, h_sotcEnabled, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|sotc
   If ( ErrorLevel )
      h_sotcEnabled := FALSE  ; default: nope

	Menu, TRAY, DeleteAll	; tabularasa
	Menu, TRAY, Click, 1	; 1-click menu
	Menu, TRAY, NoStandard
	Menu, TRAY, Add, %h_MenuWin%, h_MenuHandler
	; MultiWin mode
   If ( newID0 > 1 )
   {
      Loop, %newID0%
      {
         tmp := f_StringLeft( A_Index ": " newTit%A_Index% " [" newApp%A_Index% "]" )
        	Menu, MultiWinMenu, Add, %tmp%, h_MenuHandlerMultiWin
      }
      ; possibly ready for left click -> unhide this win
      Menu, TRAY, Add, %lng_MenuMultiWin%, :MultiWinMenu
   }
   Menu, TRAY, Add
	Menu, TRAY, Add, %lng_MenuAssignHotkey%, h_MenuHandler
	; custom name
   If ( h_cnEnabled )
	   Menu, TRAY, Add, %lng_MenuNameRemove%, h_MenuHandler
	Else
	   Menu, TRAY, Add, %lng_MenuNameChange%, h_MenuHandler
	; custom icon
   If ( h_ciEnabled )
	   Menu, TRAY, Add, %lng_MenuIconRemove%, h_MenuHandler
	Else
	   Menu, TRAY, Add, %lng_MenuIconChange%, h_MenuHandler
   Menu, TRAY, Add
	; NOT in MultiWin mode
	If ( newID0 = 1 )
	{
      ; ShowOnTitleChange
      Menu, TRAY, Add, %lng_MenuShowOnTitleChange%, h_MenuHandler
   	If ( h_sotcEnabled )
         Menu, TRAY, Check, %lng_MenuShowOnTitleChange%
      Else
         Menu, TRAY, Uncheck, %lng_MenuShowOnTitleChange%
      ; BossKey
      Menu, TRAY, Add, %lng_MenuOnBKList%, h_MenuHandler
   	If ( h_bkEnabled )
   		Menu, TRAY, Check, %lng_MenuOnBKList%
   	Else
   		Menu, TRAY, Uncheck, %lng_MenuOnBKList%
   	Menu, TRAY, Add
   }
	Menu, TRAY, Add, %lng_MenuClose%, h_MenuHandler
	Menu, TRAY, Add, %lng_MenuUnhide%, h_MenuHandler
	Menu, TRAY, Default, %lng_MenuUnhide%
	Menu, TRAY, Tip, %h_TrayTitle%
Return

h_MenuHandler:
   If ( A_ThisMenuItem = lng_MenuUnhide OR A_ThisMenuItem = h_MenuWin )
      Gosub, h_Unhide
   If ( A_ThisMenuItem = lng_MenuOnBKList )
      Gosub, h_BossKeyListToggle
   If ( A_ThisMenuItem = lng_MenuShowOnTitleChange )
      Gosub, h_ShowOnTitleChangeToggle

   ; do not execute a new command while another one is still showing a GUI
   If ( h_noMenuAction OR h_noGUIAction )
   {
      SoundPlay, *16 
      Return
   }

   ; subs creating GUIs, no interference allowed here
   h_noMenuAction := TRUE

   If ( A_ThisMenuItem = lng_MenuClose )
      Gosub, h_Close
   If ( A_ThisMenuItem = lng_MenuAssignHotkey )
      Gosub, h_SetHotkeyHelper
   ; change or remove custom name
   If ( A_ThisMenuItem = lng_MenuNameChange )
      Gosub, h_ChangeName
   If ( A_ThisMenuItem = lng_MenuNameRemove )
      Gosub, h_RemoveName
   ; change or remove custom icon
   If ( A_ThisMenuItem = lng_MenuIconChange )
      Gosub, h_ChangeIcon
   If ( A_ThisMenuItem = lng_MenuIconRemove )
      Gosub, h_RemoveIcon

   h_noMenuAction := FALSE
Return

h_MenuHandlerMultiWin:
   ; this routine unhides the selected window
   StringSplit, tmp, A_ThisMenuItem, :%A_Space%, %A_Space%
   h_WinID := newID%tmp1%
   If (h_WinID = "")
      Return
   ; unhide this particular window
   WinShow, ahk_id %h_WinID%
   WinRestore, ahk_id %h_WinID%
   WinActivate, ahk_id %h_WinID%
   ; update sub menu
  	Menu, MultiWinMenu, Delete, %A_ThisMenuItem%
   ; remove win ID from list
   newID%tmp1% =
Return

h_SetHotkeyHelper:
   h_noGUIAction := TRUE
   Suspend, On

   addWinKey =
   chkWin = 0
   newKey = %h_hkKey%

   ; strip off modifier WIN and check the box instead
   If InStr( newKey, "#" )
   {
      chkWin = 1
      StringReplace, newKey, newKey, # 
   }
      
   Gui, 2:+AlwaysOnTop -MinimizeBox -MaximizeBox
   Gui, 2:Add, Text, Section, %lng_SetupUnhideKey%:
   Gui, 2:Add, Checkbox, y+8 Section Checked%chkWin% vaddWinKey, WIN +
   Gui, 2:Add, Hotkey, ys yp-4 w180 vnewKey, %newKey%
   Gui, 2:Add, Text, xs w250, %lng_SetupHint%
   Gui, 2:Add, Button, ym Section w90 gh_2ButtonOK Default, %lng_SetupOK%
   Gui, 2:Add, Button, xs wp g2BtnCancel, %lng_SetupCancel%
   Gui, 2:Show, Center, %h_MenuWin% - %lng_WindowTitle%
Return

h_2ButtonOK:
   Gui, 2:+OwnDialogs
   Gui, 2:Submit, NoHide

   h_hkKey := f_ComposeHotkey( newKey, addWinKey, h_hkKey, "h_Unhide", "", h_Application "|" h_Class "|ch", lng_SetupUnhideKey )
   If ( h_hkKey = "INVALID_KEY" )
      Return

   Goto, 2GuiClose      
Return

2BtnCancel:
2GuiEscape:
2GuiClose:
  GUI, 2:Destroy
  Suspend, Off
  h_noGUIAction := FALSE
Return

h_BossKeyListToggle:
	; remove from/add to bosskey list and change tray menu accordingly
	If ( h_bkEnabled )
	{
		h_bkEnabled := FALSE
		RegDelete, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|bk
		Menu, TRAY, Uncheck, %lng_MenuOnBKList%
	}
	Else
	{
		h_bkEnabled := TRUE
		RegWrite, REG_SZ, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|bk, %h_bkEnabled%
		Menu, TRAY, Check, %lng_MenuOnBKList%
	}
Return

h_ShowOnTitleChangeToggle:
	; toggle setting and change tray menu accordingly
	If ( h_sotcEnabled )
	{
		h_sotcEnabled := FALSE
		RegDelete, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|sotc
		Menu, TRAY, Uncheck, %lng_MenuShowOnTitleChange%
	}
	Else
	{
		h_sotcEnabled := TRUE
		RegWrite, REG_SZ, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%|sotc, %h_sotcEnabled%
		Menu, TRAY, Check, %lng_MenuShowOnTitleChange%
	}
Return

;----------------------------------------------------------------------------
; Helper error -- subs
;

h_ExitWithParamError:
   f_ShowErrorMsg( lng_ExitWithParamError )
ExitApp, 1

;============================================================================
; function as launcher for minimizing (starter) starts here

;----------------------------------------------------------------------------
; Starter -- init
;

h_InitStarter:
   If ( h_Language = "de_DE" )
   {
      FormatTime, tmpDate, %versionDate%, dd.MM.yyyy
      lng_MenuAbout              = &ber %lng_WindowTitle%
      lng_MenuRestoreOnly        = &Alle Fenster wiederherstellen
      lng_MenuQuitRestoreAll     = &Wiederherstellen && beenden
      lng_MenuQuitOnly           = Nur &Beenden
      lng_MenuSetHotkeysStarter  = &Einstellungen...
      lng_MenuBKEditList         = &BossKey-Liste bearbeiten
      lng_BKPurgeListTitle       = BossKey-Liste
      lng_BKPurgeList            = Markieren Sie die Fenster, die aus der BossKey-Liste entfernt werden sollen
      lng_BKPurgeListReq         = Eintrge aus BossKey-Liste entfernen?
      lng_BKPurgeListRemove      = &Entfernen
      lng_TrayTitle              = %lng_WindowTitle%
      lng_About                  = %lng_WindowTitle% v%versionString% (%tmpDate%)`n`nMinimiert Fenster in den Tray-Bereich der Taskbar als kleine Icons.`nDazu den dritten Mausknopf bzw. den benutzerdefinierten Hotkey drcken.`nWeitere Funktionalitten: BossKey, Always-on-top und vertikales Maximieren.`n`nVertrieben unter den Auflagen der GPLv2.`n 2005-2006 bei Junyx / KTC^brain`n
      lng_SetupTitle             = %lng_WindowTitle% Einstellungen
      lng_SetupKey               = Hotkey zum Fenster minimieren
      lng_SetupMButtonTitle      = Dritte (mittlere) Maustaste zum Minimieren nutzen?
      lng_SetupMButtonOpt0       = Kein Bedarf!
      lng_SetupMButtonOpt1       = durch Klicken auf Titelleiste [STANDARD]
      lng_SetupMButtonOpt2       = durch Klicken (egal wohin)
      lng_SetupBkMultiWin        = BossKey setzt MultiWindows-Mode ein. [NEIN]
      lng_SetupBk2sMode          = BossKey-Nutzung erzwingt StealthMode (SM-Hotkey blendet Icons wieder ein). [NEIN]
      lng_SetupStealthSessOnly   = StealthMode nur fr aktuelle Sitzung. [JA]
      lng_SetupForcedMode        = Erzwinge immer Min- bzw. Maximierung des Fensters (ForcedMode). [NEIN]
      lng_SetupJaakonMode        = Frage immer nach einem "Eigenen Name", wenn noch keiner vergeben wurde. [NEIN]
      lng_SetupNoErrorMsgsMode   = Keinerlei Fehlermeldungen anzeigen. [NEIN]
      lng_SetupBossKey           = BossKey
      lng_SetupBKTitle           = BossKey-Modus bestimmen
      lng_SetupBKOpt0            = Opt-Out  -  minimiere alle Fenster AUSSER die der BossKey-Liste [STANDARD]
      lng_SetupBKOpt1            = Opt-In  -  minimiere NUR Fenster der BossKey-Liste
      lng_SetupStealthKey        = Hotkey fr StealthMode Umschaltung
      lng_SetupAOTKey            = Hotkey fr Always-On-Top
      lng_AOTtraytip             = Fenster ist nun immer im Vordergrund.
      lng_SetupYMaxKey           = Hotkey fr vertikales Maximieren
      lng_YMaxtraytip            = Fenster wurde vertikal maximiert.
      lng_YMaxNottraytip         = Fenster konnte nicht vertikal maximiert werden.
      lng_No3BtnMouseMsg			= Keine Maus mit 3 Knpfen gefunden. Mausuntersttzung abgeschaltet!
      ; Errors
      lng_ExitWithRunningError   = Fehler: Min2Tray wurde schon einmal gestartet!
      lng_ExitWithAHKError       = Fehler: Konnte Datei "AutoHotkey.exe" nicht ausfindig machen!
      lng_ExitWithLockError      = Fehler: Konnte Lockfile nicht lschen. Bitte Datei von Hand lschen:
   }
   Else
   {
      FormatTime, tmpDate, %versionDate%, ShortDate
      lng_MenuAbout              = &About %lng_WindowTitle%
      lng_MenuRestoreOnly        = &Restore all windows
      lng_MenuQuitRestoreAll     = Restore &all && quit
      lng_MenuQuitOnly           = &Quit only
      lng_MenuSetHotkeysStarter  = &Preferences...
      lng_MenuBKEditList         = Edit &BossKey list
      lng_BKPurgeListTitle       = BossKey list
      lng_BKPurgeList            = Select windows to be removed from BossKey list
      lng_BKPurgeListReq         = entries to remove from BossKey list. Continue?
      lng_BKPurgeListRemove      = &Remove
      lng_TrayTitle              = %lng_WindowTitle%
      lng_About                  = %lng_WindowTitle% v%versionString% (%tmpDate%)`n`nMinimize windows to the tray area of taskbar as icons.`nPress third mouse button or custom hotkey.`nAdditional features: BossKey, always-on-top and vertical maximizing.`n`nDistributed under the terms of GPLv2.`n 2005-2006 by Junyx / KTC^brain`n
      lng_SetupTitle             = %lng_WindowTitle% Prefs
      lng_SetupKey               = Hotkey to minimize window
      lng_SetupMButtonTitle      = Use third (middle) mouse button for minimizing?
      lng_SetupMButtonOpt0       = Nope!
      lng_SetupMButtonOpt1       = by clicking on titlebar [DEFAULT]
      lng_SetupMButtonOpt2       = by clicking (no matter where)
      lng_SetupBkMultiWin        = BossKey employs MultiWindows mode. [NO]
      lng_SetupBk2sMode          = Using BossKey enforces StealthMode (SM hotkey makes icons re-appear). [NO]
      lng_SetupStealthSessOnly   = StealthMode is current session only. [YES]
      lng_SetupForcedMode        = Always force min- or maximizing of window (ForcedMode). [NO]
      lng_SetupJaakonMode        = Always prompt for custom name if none is present. [NO]
      lng_SetupNoErrorMsgsMode   = Do not show any error messages. [NO]
      lng_SetupBossKey           = BossKey
      lng_SetupBKTitle           = Set BossKey mode
      lng_SetupBKOpt0            = Opt-Out  -  minimize all windows EXCEPT those from BossKey list [DEFAULT]
      lng_SetupBKOpt1            = Opt-In  -  minimize ONLY windows from BossKey list
      lng_SetupStealthKey        = Hotkey for StealthMode toggle
      lng_SetupAOTKey            = Hotkey for always-on-top
      lng_AOTtraytip             = window is now always-on-top.
      lng_SetupYMaxKey           = Hotkey for maximizing vertically
      lng_YMaxtraytip            = window is now vertically maximized.
      lng_YMaxNottraytip         = window could not be vertically maximized.
      lng_No3BtnMouseMsg			= No mouse with at 3 buttons found. Mouse support disabled!
      ; Errors
      lng_ExitWithRunningError   = Error: Min2Tray is already running!
      lng_ExitWithAHKError       = Error: Could not find file "AutoHotkey.exe"!
      lng_ExitWithLockError      = Error: Could not erase lock-file! Please erase file manually:
   }

   ; lockfile checking
   m2t_lockFileName = %A_Temp%\Min2Tray.lck
   
   IniRead, m2t_chkName, %m2t_lockFileName%, M2Tlock, Name
   IniRead, m2t_chkPID, %m2t_lockFileName%, M2Tlock, PID
   IniRead, m2t_chkDate, %m2t_lockFileName%, M2Tlock, Date

   If ( ( m2t_chkName <> "ERROR" ) And ( m2t_chkPID <> "ERROR" ) And ( m2t_chkDate <> "ERROR" ) )
   {
      Process, Exist, %m2t_chkPID%			; PID still existing...
      If ( ErrorLevel )
      {
      	Process, Exist, %m2t_chkName%		; and name still existing
      	If ( ErrorLevel )
      		Goto, ExitWithRunningError	; error&exit
      }
   }

   ; find AutoHotkey.exe and save its path to a variable for later use
   If ( Not A_IsCompiled )
   {
      RegRead, h_ahkFullPath, HKLM, SOFTWARE\AutoHotkey, InstallDir
      h_ahkFullPath = %h_ahkFullPath%\AutoHotkey.exe
      IfNotExist, %h_ahkFullPath%
      {
      	h_ahkFullPath := f_FindFile( "AutoHotkey.exe", A_ProgramFiles )
      	If ( h_ahkFullPath = "" )
      	{
      		StringSplit, tmparray, A_WinDir, \	; search windir
      		h_ahkFullPath := f_FindFile( "AutoHotkey.exe", tmparray1 )
      		If ( h_ahkFullPath = "" )
      			Goto, ExitWithAHKError
      	}
      }
   }

   ; number of mouse buttons
	SysGet, tmp, 43
   If ( tmp < 3 )
   {
      f_ShowErrorMsg( lng_No3BtnMouseMsg )
      m2t_noMButton := TRUE
   }
   Else
		m2t_noMButton := FALSE

   h_winPIDs :=  ; h_winPIDs%WIN_ID% -- array for WIN_IDs and corresponding PIDs
   h_helperPIDs :=	; h_helperPIDs%h_hPIDcount% -- array for PIDs of started helpers
   h_hPIDcount  := 0

   Menu, TRAY, NoStandard
   Menu, TRAY, Add, %lng_MenuAbout%, m2t_MenuHandler
   Menu, TRAY, Add
   Menu, TRAY, Add, %lng_MenuBKEditList%, m2t_MenuHandler
   Menu, TRAY, Add, %lng_MenuSetHotkeysStarter%, m2t_MenuHandler
   Menu, TRAY, Add
   Menu, TRAY, Add, %lng_MenuRestoreOnly%, m2t_MenuHandler
   Menu, TRAY, Add, %lng_MenuQuitRestoreAll%, m2t_MenuHandler
   Menu, TRAY, Add, %lng_MenuQuitOnly%, m2t_MenuHandler
   Menu, TRAY, Tip, %lng_TrayTitle%

   ; assigning keys
   ; key for minimizing
   m2t_hkKey =
   RegRead, m2t_hkKey, HKCU, %h_RegSubkey%\Starter, HotkeyMinimize
   f_AssignHotkey( m2t_hkKey, "h_StartNewHelper" )
   ; stealthmode key
   m2t_hkStealth =
	RegRead, m2t_hkStealth, HKCU, %h_RegSubkey%\Starter, HotkeyStealth
	If ( ErrorLevel OR m2t_hkStealth = "" )
	   m2t_hkStealth = ^#!PgUp   ; default: Ctrl+Win+Alt+PgUp
	f_AssignHotkey( m2t_hkStealth, "h_StealthModeToggle" )
	; get bosskey mode
	RegRead, bkOptIn, HKCU, %h_RegSubkey%\Misc, BossKeyOptIn
	If ( ErrorLevel OR Not bkOptIn )
	   bkOptIn := FALSE	; default: Opt-Out mode
	Else
	   bkOptIn := TRUE
   ; bosskey
   m2t_hkBossKey =
	RegRead, m2t_hkBossKey, HKCU, %h_RegSubkey%\Starter, HotkeyBossKey
	f_AssignHotkey( m2t_hkBossKey, "h_BossKey" )
   ; y-maximizer key
   m2t_hkYMax =
	RegRead, m2t_hkYMax, HKCU, %h_RegSubkey%\Starter, HotkeyYMax
	f_AssignHotkey( m2t_hkYMax, "h_YMaxWindow" )
   ; always-on-top key
   m2t_hkAOT =
	RegRead, m2t_hkAOT, HKCU, %h_RegSubkey%\Starter, HotkeyAOT
	f_AssignHotkey( m2t_hkAOT, "h_AlwaysOnTop" )
   ; middle mouse button
   m2t_hkMButton := 0	; default: set mbutton-action off
   If ( Not m2t_noMButton )
   {
	   RegRead, m2t_hkMButton, HKCU, %h_RegSubkey%\Starter, HotkeyMButton
	   If ( m2t_hkMButton = 1 OR m2t_hkMButton = 2 )
	      f_AssignHotkey( "MButton", "h_StartNewHelper" )
   }
   ; bosskey MultiWin mode
	RegRead, m2t_BkMultiWin, HKCU, %h_RegSubkey%\Starter, BossKeyMultiWin
	If ( m2t_BkMultiWin )
	   m2t_BkMultiWin := TRUE
	Else
      m2t_BkMultiWin := FALSE
   ; bosskey to stealth mode
	RegRead, m2t_Bk2sMode, HKCU, %h_RegSubkey%\Starter, BossKeyToStealth
	If ( m2t_Bk2sMode )
	   m2t_Bk2sMode := TRUE
	Else
      m2t_Bk2sMode := FALSE
   ; stealth mode current session only
	RegRead, m2t_SmSessOnly, HKCU, %h_RegSubkey%\Starter, StealthSessionOnly
	If ( m2t_SmSessOnly OR ErrorLevel )   ; default: YES
	   m2t_SmSessOnly := TRUE
	Else
      m2t_SmSessOnly := FALSE
   Gosub, h_UpdateSessionOnlyState
   ; forced mode
	RegRead, m2t_ForcedMode, HKCU, %h_RegSubkey%\Starter, ForcedMode
	If ( m2t_ForcedMode )
	   m2t_ForcedMode := TRUE
	Else
      m2t_ForcedMode := FALSE
   ; show no error messages mode
	RegRead, m2t_NoErrorMsgs, HKCU, %h_RegSubkey%\Misc, NoErrorMessages
	If ( m2t_NoErrorMsgs )
	   m2t_NoErrorMsgs := TRUE
	Else
	   m2t_NoErrorMsgs := FALSE

   ; create temporary lockfile
   ; info: WinGet can only retrieve the ProcessName after setting up tray
   Process, Exist, 
   m2t_ownPID = %ErrorLevel%
   WinGet, m2t_ownName, ProcessName, ahk_pid %m2t_ownPID%
   
   IniWrite, %m2t_ownName%, %m2t_lockFileName%, M2Tlock, Name
   IniWrite, %m2t_ownPID%, %m2t_lockFileName%, M2Tlock, PID
   IniWrite, %A_Now%, %m2t_lockFileName%, M2Tlock, Date

	OnExit, h_QuitStarter

   ; finally: show menu/icon
   f_ShowTrayIcon( h_StarterIcon, h_StarterIcon# )
Return

m2t_MenuHandler:
   If ( A_ThisMenuItem = lng_MenuRestoreOnly )
      Gosub, h_RestoreStarter

   ; do not execute a new command while another one is still showing a GUI
   If ( m2t_noMenuAction OR m2t_noGUIAction )
   {
      SoundPlay, *16 
      Return
   }

   ; subs creating GUIs, no interference allowed here
   m2t_noMenuAction := TRUE
   
   If ( A_ThisMenuItem = lng_MenuAbout )
      Gosub, h_AboutStarter
   If ( A_ThisMenuItem = lng_MenuBKEditList )
      Gosub, h_BossKeyPurgeList
   If ( A_ThisMenuItem = lng_MenuSetHotkeysStarter )
      Gosub, h_SetHotkeyStarter
   If ( A_ThisMenuItem = lng_MenuQuitRestoreAll )
      Gosub, h_QuitRestoreStarter
   If ( A_ThisMenuItem = lng_MenuQuitOnly )
      Gosub, h_QuitStarter

   m2t_noMenuAction := FALSE
Return

h_StartNewHelper:
   If ( A_ThisHotkey = m2t_hkKey OR m2t_hkMButton = 2 ) ; what key triggered the action?
      f_StartNewHelper( "ASAP" )  ; no check for title-click
   Else
      f_StartNewHelper( "MOUSE" )
Return

h_AboutStarter:
   Msgbox, 64, %lng_WindowTitle%, %lng_About%
Return

h_RestoreStarter:	; close all helpers -> unhide every window
   Loop
   {
      If ( h_hPIDcount = 0 )
      	Break
      h_process := h_helperPIDs%h_hPIDcount%
      Process, Exist, %h_process%
      If ( ErrorLevel )	; helper still exists
      {
      	WinClose, ahk_pid %h_process%
      }
      h_hPIDcount--
      Sleep, 50
   }
Return

h_QuitRestoreStarter:
   Gosub, h_RestoreStarter
   ; ...
h_QuitStarter:
   Gosub, h_UpdateSessionOnlyState
   Gosub, h_CleanUpLockFile
ExitApp

h_UpdateSessionOnlyState:
   If ( m2t_SmSessOnly )
	   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Misc, StealthMode, 0
Return

h_CleanUpLockFile:
   IfNotExist, %m2t_lockFileName%
      Return
   
   FileDelete, %m2t_lockFileName%
   If ( ErrorLevel )
   {
      FileSetAttrib, -RHS, %m2t_lockFileName%	; one more try...
      FileDelete, %m2t_lockFileName%
      If ( ErrorLevel )
      {	; fuck it! and exit with code "2"
         tmp = %lng_ExitWithLockError% "%m2t_lockFileName%".
         f_ShowErrorMsg( tmp )
      	ExitApp, 2
      }
   }
Return

h_SetHotkeyStarter:
   m2t_noGUIAction := TRUE
   Suspend, On

   chkWinHK = 0
   chkWinBK = 0
   chkWinSK = 0
   chkWinAK = 0
   chkWinYK = 0
   newKey = %m2t_hkKey%
   newBossKey = %m2t_hkBossKey%
   newStealthKey = %m2t_hkStealth%
   newAOTKey = %m2t_hkAOT%
   newYMaxKey = %m2t_hkYMax%

   ; strip off modifier WIN from hotkey and check the box instead
   If InStr( newKey, "#" )
   {
      chkWinHK = 1
      StringReplace, newKey, newKey, # 
   }
   ; strip off modifier WIN from bosskey and check the box instead
   If InStr( newBossKey, "#" )
   {
      chkWinBK = 1
      StringReplace, newBossKey, newBossKey, # 
   }
   ; strip off modifier WIN from stealth-key and check the box instead
   If InStr( newStealthKey, "#" )
   {
      chkWinSK = 1
      StringReplace, newStealthKey, newStealthKey, # 
   }
   ; strip off modifier WIN from AOT-key and check the box instead
   If InStr( newAOTKey, "#" )
   {
      chkWinAK = 1
      StringReplace, newAOTKey, newAOTKey, # 
   }
   ; strip off modifier WIN from YMax-key and check the box instead
   If InStr( newYMaxKey, "#" )
   {
      chkWinYK = 1
      StringReplace, newYMaxKey, newYMaxKey, # 
   }
   
   Gui, +AlwaysOnTop -MinimizeBox -MaximizeBox
   ; --- left hand site
   ; hint
   Gui, Add, Text, ym w250 R3 Section, %lng_SetupHint% 
   ; hotkey + mbutton + forcedmode
   Gui, Add, Text, xs Section, %lng_SetupKey%:
   Gui, Add, Checkbox, Section y+8 Checked%chkWinHK% vaddWinKeyHK, WIN +
   Gui, Add, Hotkey, ys yp-4 w180 vnewKey, %newKey%
   ; bosskey
   Gui, Add, Text, xs Y+14 Section, %lng_SetupBossKey%:
   Gui, Add, Checkbox, Section y+8 Checked%chkWinBK% vaddWinKeyBK, WIN +
   Gui, Add, Hotkey, ys yp-4 w180 vnewBossKey, %newBossKey%
   ; stealth-key
   Gui, Add, Text, xs Y+14 Section, %lng_SetupStealthKey%:
   Gui, Add, Checkbox, Section y+8 Checked%chkWinSK% vaddWinKeySK, WIN +
   Gui, Add, Hotkey, ys yp-4 w180 vnewStealthKey, %newStealthKey%
   ; AOT-key
   Gui, Add, Text, xs Y+14 Section, %lng_SetupAOTKey%:
   Gui, Add, Checkbox, Section y+8 Checked%chkWinAK% vaddWinKeyAK, WIN +
   Gui, Add, Hotkey, ys yp-4 w180 vnewAOTKey, %newAOTKey%
   ;Gui, Add, Text, xs w250 R3, %lng_SetupHint% 
   ; YMax-key
   Gui, Add, Text, xs Y+14 Section, %lng_SetupYMaxKey%:
   Gui, Add, Checkbox, Section y+8 Checked%chkWinYK% vaddWinKeyYK, WIN +
   Gui, Add, Hotkey, ys yp-4 w180 vnewYMaxKey, %newYMaxKey%
   ; --- right hand site
   ; mbutton options
   If ( m2t_noMButton )
		tmpDis = Disabled
	Else
   	tmpDis =
   tmp0 = 0
   tmp1 = 0
   tmp2 = 0
   tmp%m2t_hkMButton% = 1 ; check the active radiobox
   Gui, Add, Text, ym Section w250 %tmpDis%, %lng_SetupMButtonTitle%
   Gui, Add, Radio, xs w230 vnewMButton0 %tmpDis% Checked%tmp0%, %lng_SetupMButtonOpt0%
   Gui, Add, Radio, xs w230 vnewMButton1 %tmpDis% Checked%tmp1%, %lng_SetupMButtonOpt1%
   Gui, Add, Radio, xs w230 vnewMButton2 %tmpDis% Checked%tmp2%, %lng_SetupMButtonOpt2%
   ; bosskey options
   tmp0 = 0
   tmp1 = 0
   tmp%bkOptIn% = 1 ; check the active radiobox
   Gui, Add, Text, xs w250 Y+20, %lng_SetupBKTitle%
   Gui, Add, Radio, xs w230 vnewBKmode0 Checked%tmp0%, %lng_SetupBKOpt0%
   Gui, Add, Radio, xs w230 vnewBKmode1 Checked%tmp1%, %lng_SetupBKOpt1%
   ; on/off options
   Gui, Add, Checkbox, xs Y+20 w250 vnewBkMultiWin Checked%m2t_BkMultiWin%, %lng_SetupBkMultiWin%
   Gui, Add, Checkbox, xs Y w250 vnewBk2sMode Checked%m2t_Bk2sMode%, %lng_SetupBk2sMode%
   Gui, Add, Checkbox, xs Y w250 vnewSmSessOnly Checked%m2t_SmSessOnly%, %lng_SetupStealthSessOnly%
   Gui, Add, Checkbox, xs Y w250 vnewForcedMode Checked%m2t_ForcedMode%, %lng_SetupForcedMode%
   Gui, Add, Checkbox, xs Y w250 vnewJaakonMode Checked%m2t_JaakonMode%, %lng_SetupJaakonMode%
   Gui, Add, Checkbox, xs Y w250 vnewNoErrorMsgs Checked%m2t_NoErrorMsgs%, %lng_SetupNoErrorMsgsMode%
   ; --- ok + cancel
   Gui, Add, Button, ym Section w90 gh_ButtonOK Default, %lng_SetupOK%
   Gui, Add, Button, xs wp gBtnCancel, %lng_SetupCancel%
   
   Gui, Show, Center, %lng_SetupTitle%
Return

h_ButtonOK:
   Gui, +OwnDialogs
   Gui, Submit, NoHide

   ; hotkey for minimizing
   m2t_hkKey := f_ComposeHotkey( newKey, addWinKeyHK, m2t_hkKey, "h_StartNewHelper", "Starter", "HotkeyMinimize", lng_SetupKey )
   If ( m2t_hkKey = "INVALID_KEY" )
      Return

   ; hotkey for always-on-top
   m2t_hkAOT := f_ComposeHotkey( newAOTKey, addWinKeyAK, m2t_hkAOT, "h_AlwaysOnTop", "Starter", "HotkeyAOT", lng_SetupAOTKey )
   If ( m2t_hkAOT = "INVALID_KEY" )
      Return

   ; hotkey for y-maximizing
   m2t_hkYMax := f_ComposeHotkey( newYMaxKey, addWinKeyYK, m2t_hkYMax, "h_YMaxWindow", "Starter", "HotkeyYMax", lng_SetupYMaxKey )
   If ( m2t_hkYMax = "INVALID_KEY" )
      Return

   ; hotkey for stealth mode
   m2t_hkStealth := f_ComposeHotkey( newStealthKey, addWinKeySK, m2t_hkStealth, "h_StealthModeToggle", "Starter", "HotkeyStealth", lng_SetupStealthKey )
   If ( m2t_hkStealth = "INVALID_KEY" )
      Return

   ; bosskey
   m2t_hkBossKey := f_ComposeHotkey( newBossKey, addWinKeyBK, m2t_hkBossKey, "h_BossKey", "Starter", "HotkeyBossKey", lng_SetupBossKey )
   If ( m2t_hkBossKey = "INVALID_KEY" )
      Return

   ; bosskey mode
   If ( newBKmode1 ) ; opt-in
      bkOptIn := TRUE
   Else ; default: opt-out
      bkOptIn := FALSE 
   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Misc, BossKeyOptIn, %bkOptIn%

   ; mbutton mode
   If ( Not m2t_noMButton )
	{
	   If ( newMButton1 ) ; title-click
	      m2t_hkMButton := 1
	   Else If ( newMButton2 ) ; just third mouse button
	      m2t_hkMButton := 2
	   Else ; set mbutton-action off
	      m2t_hkMButton := 0 
	   If ( m2t_hkMButton = 1 OR m2t_hkMButton = 2 )
	      f_AssignHotkey( "MButton", "h_StartNewHelper" )
	   Else ; set mbutton-action off
	      f_AssignHotkey( "MButton" )
	   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Starter, HotkeyMButton, %m2t_hkMButton%
	}

   ; bosskey MultiWin mode
	If ( newBkMultiWin )
	   m2t_BkMultiWin := TRUE
	Else
	   m2t_BkMultiWin := FALSE
   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Starter, BossKeyMultiWin, %m2t_BkMultiWin%

   ; bosskey to stealth mode
	If ( newBk2sMode )
	   m2t_Bk2sMode := TRUE
	Else
	   m2t_Bk2sMode := FALSE
   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Starter, BossKeyToStealth, %m2t_Bk2sMode%

   ; stealth mode is current session only 
	If ( newSmSessOnly )
	   m2t_SmSessOnly := TRUE
	Else
	   m2t_SmSessOnly := FALSE
   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Starter, StealthSessionOnly, %m2t_SmSessOnly%

   ; forced mode
	If ( newForcedMode )
	   m2t_ForcedMode := TRUE
	Else
	   m2t_ForcedMode := FALSE
   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Starter, ForcedMode, %m2t_ForcedMode%

   ; Jaakon's mode
	If ( newJaakonMode )
	   m2t_JaakonMode := TRUE
	Else
	   m2t_JaakonMode := FALSE
   RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Misc, JaakonMode, %m2t_JaakonMode%

   ; show no error messages mode
	If ( newNoErrorMsgs )
	   m2t_NoErrorMsgs := TRUE
	Else
	   m2t_NoErrorMsgs := FALSE
	RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Misc, NoErrorMessages, %m2t_NoErrorMsgs%

   Goto, GuiClose
Return

BtnCancel:
GuiEscape:
GuiClose:
   GUI, Destroy
   Suspend, Off
   m2t_noGUIAction := FALSE
Return

h_BossKey:
   If ( m2t_Bk2sMode )
   {  ; enforce StealthMode
   	RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Misc, StealthMode, 1
   	f_ShowTrayIcon( h_StarterIcon, h_StarterIcon# )
   }
   f_BossKey()
Return

h_BossKeyPurgeList:
   m2t_noGUIAction := TRUE
   Suspend, On

   regList =
   boxList =
   idx = 0
	Loop, HKCU, %h_RegSubkey%
	{
	   StringSplit, tmp, A_LoopRegName, |
		If ( tmp0 = 3 AND tmp3 = "bk" )
		{
		   idx++
		   regList%idx% = %A_LoopRegName%
			boxList = %boxList%%tmp1% (%tmp2%)`n
		}
		Else If ( tmp0 = 2 AND tmp2 = "bk" )
		{
		   idx++
		   regList%idx% = %A_LoopRegName%
			boxList = %boxList%%tmp1%`n
		}
	}

   Gui, 4:Default
   Gui, +AlwaysOnTop -MinimizeBox -MaximizeBox +Delimiter`n
   Gui, Add, Text, Section, %lng_BKPurgeList%:
   Gui, Add, ListBox, Section xs r15 wp+20 8 AltSubmit vboxChoice  t32 t64 t64, %boxList%
   Gui, Add, Button, ys Section w90 Default g4BtnOK, %lng_BKPurgeListRemove%
   Gui, Add, Button, xs wp g4GuiClose, %lng_SetupCancel%
   Gui, Show, Center, %lng_BKPurgeListTitle% 
Return

4BtnOK:
   Gui, +OwnDialogs
   Gui, Submit, NoHide

   delCount = 0
   Loop, PARSE, boxChoice, `n
   {
      If ( A_LoopField <= idx )
      {
         delCount++
         ; re-use and re-fill array boxList here
         boxList%delCount% := regList%A_LoopField%
      }
   }

   If ( delCount )
   {
      Msgbox, 36, %lng_BKPurgeListTitle%, %delCount% %lng_BKPurgeListReq%
         IfMsgBox, No
            Return
   } 

   Loop, %delCount%
   {
      tmp := boxList%A_Index%
      RegDelete, HKCU, %h_RegSubkey%, %tmp%
   }
   ; ...
4GuiEscape:
4GuiClose:
   GUI, Destroy
   Suspend, Off
   m2t_noGUIAction := FALSE
Return

h_StealthModeToggle:
	RegRead, tmp, HKCU, %h_RegSubkey%\Misc, StealthMode
	tmp := Not tmp
	RegWrite, REG_SZ, HKCU, %h_RegSubkey%\Misc, StealthMode, %tmp%
	; update/remove icon for Starter
	f_ShowTrayIcon( h_StarterIcon, h_StarterIcon# )
Return

h_YMaxWindow:
	f_YMaxWindow()
Return

h_AlwaysOnTop:
   f_AlwaysOnTop()
Return

;-----------------------------------------------------------------------------
; Starter functions
;

f_BossKey()
{  ; hide all visible windows with one keystroke
   Global bkOptIn, h_RegSubkey, m2t_hkBossKey, m2t_BkMultiWin
   
	; build list with currently active (visible) windows
	DetectHiddenWindows, Off
	WinGet, bkWinID, List
	DetectHiddenWindows, On
	If ( bkWinID = 0 )  ; no windows to hide
	   Return

	; go thru list and hide all windows, except those that are opt'ed-out by user
	; bkHideID holds the count of windows to hide, bkHideID1 is the first...
	bkHideID = 0
	Loop, %bkWinID%
	{
		tmpWinID := bkWinID%A_Index%
		WinGetClass, bkClass, ahk_id %tmpWinID%
		If bkClass in Shell_TrayWnd,Progman,WorkerW,TfPSPad  ;BaseBar	; skip these classes
		   Continue
		WinGet, bkApplication, ProcessName, ahk_id %tmpWinID%
		RegRead, bkEnabled, HKCU, %h_RegSubkey%, %bkApplication%|%bkClass%|bk	; app+class
		If ( Not bkEnabled )
			RegRead, bkEnabled, HKCU, %h_RegSubkey%, %bkApplication%|bk 	; generic app (without class)

      ; remember id of window if not already done
      If ( Not bkReactivateID )
         bkReactivateID = %tmpWinID%

		; opt-out mode: hide all windows, BUT user-specified ones
		; opt-in mode : hide ONLY user-specified windows
		; hide if ((optin AND window) OR (optout AND not-window))
		If ( ( bkOptIn AND bkEnabled ) OR ( Not bkOptIn AND Not bkEnabled ) )
		{
		   bkHideID++
		   bkHideID%bkHideID% := tmpWinID
      	WinMinimize, ahk_id %tmpWinID%

		   ; reset remembered id if win is the one that gets hidden
		   If ( bkReactivateID = tmpWinID )
		      bkReactivateID = 0
		}
	}
   ; make remembered window active again
   If ( bkReactivateID )
   {
      Sleep, 75
      IfWinNotActive, ahk_id %bkReactivateID%
         WinActivate
   }

   ; go and hide windows
   If ( bkHideID = 0 )
      Return   ; nothing to hide, pal
   Else If ( m2t_BkMultiWin )
   {  ; hide to ONE icon
      f_StartNewHelper( "RESET", WinID )
      Loop, %bkHideID%
      {
         WinID := bkHideID%A_Index%
         f_StartNewHelper( "ADD", WinID )
      }
      f_StartNewHelper( "SUBMIT", "BossKey'ed" )
   }
   Else
   {  ; hide to MULTIPLE icons
      Loop, %bkHideID%
      {
         WinID := bkHideID%A_Index%
         f_StartNewHelper( "ASAP", WinID )
      }
   }
}

f_StartNewHelper( listCmd="", winID=FALSE )
{  ; returns TRUE on succes or the list of window IDs ("GET")
   Global  ; assumption: all vars are global except that are declared Local
   Local h_CaptionHeight, h_BorderHeight, h_MouseX, h_MouseY, h_WindowWidth
   Local h_WinStyle, h_ID, h_newPID, cmdLine, winCount, max, helperTitle
   Static listID  ; listID: list of memorized window IDs seperated by "|"

   ; commands
   If ( listCmd = "SUBMIT" ) {
      If ( listID = "" )
         Return   ; nothing on list
      If ( winID )
      {  ; cut title to 25 chars, max.
         StringLeft, tmp, winID, 25
         helperTitle = "%tmp%"
      }
      Else
         helperTitle = "MultiWindows"
      winID = %listID%
      Gosub, h_suStartNewHelper1
      Return TRUE 
   }
   Else If ( listCmd = "RESET" ) {
      listID =
      Return TRUE
   }
   Else If ( listCmd = "GET" )
      Return listID

   ; check window ID
	If ( winID ) {
      IfWinNotExist, ahk_id %winID%	; set "last used window"
         Return FALSE
	} Else {
      WinWait, A,, 1			; set "last used window"
      If ( ErrorLevel )	; It timed out, so do nothing
         Return FALSE
      WinGet, winID, ID
	}
   WinGet, winStyle, Style ; winID found, lastfoundwin set
   If ( m2t_ForcedMode )
      cmpWinStyle = 0x10000000   ; ( WS_VISIBLE )
   Else
      cmpWinStyle = 0x10020000   ; ( WS_VISIBLE | WS_MINIMIZEBOX )
   If Not ( winStyle & cmpWinStyle = cmpWinStyle )
      Return FALSE   ; window style not suitable for hiding

   ; commands needing "winID"
   If ( listCmd = "ADD" ) {
      listID = %listID%%winID%|
      Return TRUE ; right now do nothing but adding
   }
   Else If ( listCmd = "ASAP" ) {
      Gosub, h_suStartNewHelper1  ; minimize immedieatly
      Return TRUE
   }
   Else If ( listCmd = "MOUSE" ) { ; otherwise check if middle-click on title-bar
      CoordMode, Mouse, Relative
      SysGet, h_CaptionHeight, 4	; SM_CYCAPTION
      SysGet, h_BorderHeight, 7	; SM_CXDLGFRAME
      MouseGetPos, h_MouseX, h_MouseY
      If (( h_MouseY > 0 ) And ( h_MouseY < h_CaptionHeight + h_BorderHeight )) {
         WinGetPos, , , h_WindowWidth
         If (( h_MouseX > 0 ) And ( h_MouseX < h_WindowWidth ))
            Gosub, h_suStartNewHelper1
            Return TRUE
      } Else
         MouseClick, Middle
   }
   Return FALSE

   ; sub-routine in function
   h_suStartNewHelper1:
      ; if there are more then "max" window ID, spawn another helper
      ; this hopefully avoids too long command lines
      max = 15
      StringSplit, tmpID, winID, |
      winCount = 0
      Loop, %tmpID0%
      {
         tmp := tmpID%A_Index%
         If ( tmp <> "" )
         {
            cmdline = %cmdline% %tmp%
            winCount++
         }
         If ( winCount = max )
         {
            Gosub, h_suStartNewHelper2
            winCount = 0
            cmdline =
         }
      }
      ; if there was some "left-over"
      If ( winCount > 0 )
         Gosub, h_suStartNewHelper2

      /*  OLD STUFF
      ; does helper for this win ID exist
      h_process := h_winPIDs%h_ID%  ; get PID of helper for WIN_ID if already started
      If ( h_process <> "" ) {
         Process, Exist, %h_process%
         If ( ErrorLevel )	; helper still exists
            Return          ; -> do not minimize window again
      }
      ; -> otherwise: start new helper
      h_winPIDs%h_ID% :=  ; clear var with new WIN_ID
      */
   Return

   ; another sub routine just for spawning the helper
   h_suStartNewHelper2:
      If ( winCount > 1 )
         cmdLine = %helperTitle% %cmdLine%

      If ( A_IsCompiled )
         Run, "%A_ScriptFullPath%" %cmdLine%, , UseErrorLevel, h_newPID
      Else
      	Run, "%h_ahkFullPath%" "%A_ScriptFullPath%" %cmdLine%, , UseErrorLevel, h_newPID
      
      If ( ErrorLevel = "ERROR" )	; helper could not be started
      	Return
      
      ;h_winPIDs%h_ID% = %h_newPID% ; assign new PID
      h_hPIDcount++
      h_helperPIDs%h_hPIDcount% = %h_newPID%
   Return   
}

f_YMaxWindow()
{	; maximize the active window vertically
	Global lng_YMaxtraytip, lng_YMaxNottraytip, m2t_ForcedMode

   IfWinNotActive, A
      Return

   WinGet, max, MinMax
   If ( max )
      Return   ; do not y-maximize windows that are already maximized by system

   SysGet, mon, MonitorWorkArea
	WinGetPos, , curY, , curH
	If ( curY=monTop AND curH=monBottom )
		Return	; do not y-max if window is already maxed by this script

   ; prepare title for traytip
	WinGetActiveTitle, title
	If StrLen(title) > 50
	{
	   StringLeft, title, title, 47
	   title = %title%...
	}

   If ( Not m2t_ForcedMode )
   {  ; do not force maximization
   	WinGet, winStyle, Style
   	If ( winStyle & 0x40000 <> 0x40000 )
   	{
      	txt = "%title%":`n%lng_YMaxNottraytip%
      	f_TrayTip(txt)
   		Return	; do not y-max if window has no WS_SIZEBOX style
   	}
   }
   
   WinMove, , , , %monTop%, , %monBottom%
	txt = "%title%":`n%lng_YMaxtraytip%
	f_TrayTip(txt)

	Return
}

f_AlwaysOnTop()
{  ; make active window "always on top" or remove
   ; "aot" from window if this style is already present,
   ; indicate "aot" by doing some fancy stuff to the window
   Global lng_AOTtraytip
      
   WinWait, A,, 1			; set "last used window"
   If ( ErrorLevel )	; It timed out, so do nothing
      Return
      
   ; Desktop and Tray cannot be made AOT
   WinGetClass, class
   If class in Shell_TrayWnd,Progman,WorkerW
   	Return

   WinGet, xstyle, ExStyle
   If (xstyle & 0x8)  ; 0x8 is WS_EX_TOPMOST.
   {
      ; restore tranxparency
      WinSet, Transparent, 255
      Sleep, 50
      WinSet, Transparent, Off
      ;WinSet, Region

      WinSet, AlwaysOnTop, Off
      f_TrayTip("")
   }
   Else
   {
      WinGetActiveTitle, title
      WinSet, AlwaysOnTop, On
      ; set window to slight transparency
      WinSet, Transparent, 210

/* this was my first try to indicate that a window is always-on-top.
   unfortunatly the zigzag line within the caption vanishes when
   the user minimizes the window.
   this is very annoying but may be a restriction of the windowing system.

      WinGetPos, , , width
      SysGet, caption_height, 4	; SM_CYCAPTION
      SysGet, border_width, 32   ; SM_CXSIZEFRAME
      SysGet, border_height, 33  ; SM_CYSIZEFRAME: Thickness of the sizing border around the perimeter of a window

      ; first:  draw region around complete window, with max. width and height
      sw := A_ScreenWidth + 5
      sh := A_ScreenHeight + 5

      ; second: drill 'hole' into this region (fancy style)
      thck := caption_height / 3 ; thickness of line (default: 6)
      y1 := border_height -1
      y2 := border_height + caption_height - thck -1 ; "-1" is bottom/top offset on title bar
      y3 := y1 + thck
      y4 := y2 + thck
      start := border_width + width / 5
      length := width / 5 * 3 / 8 ; zigzag will occupie 3/5 of title bar length
      Loop, 9
      {
         x%A_Index% := start + (length * (A_Index - 1)) 
      }
      ; set the fancy zigzag
      WinSet, Region, 0-0 %sw%-0 %sw%-%sh% 0-%sh% 0-0  %x1%-%y1% %x2%-%y2% %x3%-%y1% %x4%-%y2% %x5%-%y1% %x6%-%y2% %x7%-%y1% %x8%-%y2% %x9%-%y1% %x9%-%y3% %x8%-%y4% %x7%-%y3% %x6%-%y4% %x5%-%y3% %x4%-%y4% %x3%-%y3% %x2%-%y4% %x1%-%y3% %x1%-%y1%
*/

      ; show balloon info
      If StrLen(title) > 50
      {
         StringLeft, title, title, 47
         title = %title%...
      }
      txt = "%title%":`n%lng_AOTtraytip%
      f_TrayTip(txt)
   }
   Return
}

;-----------------------------------------------------------------------------
; Starter error -- subs
;

ExitWithAHKError:
   f_ShowErrorMsg( lng_ExitWithAHKError )
ExitApp, 1

ExitWithRunningError:
   f_ShowErrorMsg( lng_ExitWithRunningError )
ExitApp, 1

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; general -- subs
;

h_InitGeneral:
   ; key in HKCU to store stuff under
   h_RegSubkey = Software\KTC\Min2Tray
   
   ; l10n
   h_Language =
   StringRight, tmp, A_Language, 2
   If ( tmp = 07 )	; = Deutsch (0407, 0807, 0c07 ...)
      h_Language = de_DE
    
   If ( h_Language = "de_DE" )
   {
      lng_WindowTitle            = Min2Tray
      lng_SetupOK                = &bernehmen
      lng_SetupCancel            = &Abbrechen
      lng_SetupHint              = Hinweis: Drcken Sie ENTFERNEN und besttigen Sie mit "%lng_SetupOK%", um einen Hotkey zu lschen!
      lng_SetupInvalidHotkey1    = Folgender Hotkey konnte nicht zugewiesen werden: "
      lng_SetupInvalidHotkey2    = ".`nBitte anderen Hotkey whlen oder den Hotkey entfernen!
      ; Errors
      lng_ExitWithOSError        = Fehler: Falsches Betriebssystem. Bentige Windows NT und hher!
      lng_ExitWithVersionError   = Fehler: Bentige "AutoHotkey.exe" Version 1.0.43.09 oder hher!
   }
   Else
   {
      lng_WindowTitle            = Min2Tray
      lng_SetupOK                = &Apply
      lng_SetupCancel            = &Cancel
      lng_SetupHint              = Hint: Press DELETE on keyboard and confirm by clicking "%lng_SetupOK%" to clear a hotkey!
      lng_SetupInvalidHotkey1    = The following hotkey could not be assigned: "
      lng_SetupInvalidHotkey2    = ".`nPlease use a different hotkey or delete the hotkey!
      ; Error
      lng_ExitWithOSError        = Error: Wrong operating system. Need Windows NT or newer!
      lng_ExitWithVersionError   = Error: Need AutoHotkey.exe version 1.0.43.09 or newer!
   }

   ; some sanity checks   
   If ( A_OSType <> "WIN32_NT" )
   {
      f_ShowErrorMsg( lng_ExitWithOSError )
      ExitApp, 1
   }
   If ( A_AhkVersion < "1.0.43.11" )
   {
      f_ShowErrorMsg( lng_ExitWithVersionError )
      ExitApp, 1
   }

   ; user defined/ default tray icon for starter and as default helper icon
   h_StarterIcon  = %A_ScriptDir%\Min2Tray.ico
   h_StarterIcon# = 1
   If Not FileExist( h_StarterIcon )
   {
      If ( A_IsCompiled )
         h_StarterIcon  = %A_ScriptFullPath%
      Else
      {
         h_StarterIcon  = %A_WinDir%\system32\shell32.dll
         h_StarterIcon# = 175 ; 25
      }
   }

   ; --- get registry settings
   ; Jaakon's mode - prompt for custom name if none present
	RegRead, m2t_JaakonMode, HKCU, %h_RegSubkey%\Misc, JaakonMode
	If ( m2t_JaakonMode )
	   m2t_JaakonMode := TRUE
	Else
      m2t_JaakonMode := FALSE
   ; DontBugMe -- switch off all notifications (tray tip, tool tip)
	RegRead, m2t_DontBugMe, HKCU, %h_RegSubkey%\Misc, DontBugMe
	If ( m2t_DontBugMe )
	   m2t_DontBugMe := TRUE
	Else
      m2t_DontBugMe := FALSE
Return

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; general -- functions
;

f_ShowErrorMsg( error_text="" )
{  ; show window with an error message
   Global lng_WindowTitle, h_RegSubkey

   If ( error_text = "" )
      Return

   ; skip showing if registry key is set
	RegRead, show_msg, HKCU, %h_RegSubkey%\Misc, NoErrorMessages
	If ( Not show_msg )
      Msgbox, 48, %lng_WindowTitle%, %error_text%
}

f_FindFile( name, startPath )
{
   StringRight, tmp, startPath, 1
   If ( ( tmp <> "" ) and ( tmp <> "\" ) )
      startPath = %startPath%\
   ; start file search
   Loop, %startPath%%name%, 0, 1
   {
      appExe = %A_LoopFileLongPath%
      IfExist, %appExe%
         Break		; exit with first found app or ""
   }
   Return %appExe%
}

f_ShowTrayIcon( file, number )
{
   Global h_RegSubkey, h_StarterIcon, h_StarterIcon#

   Menu, TRAY, UseErrorLevel
	RegRead, stealth, HKCU, %h_RegSubkey%\Misc, StealthMode	; are we in StealthMode?
	If ( Not ErrorLevel )
		If ( stealth )
		{
			If ( Not A_IconHidden )
				Menu, TRAY, NoIcon
			Return
		}

	If ( Not A_IconHidden )	; coming from hidden icon: ALWAYS renew icon!
		If ( A_IconFile = file AND A_IconNumber = number )	; icon props not changed
			Return

   IfExist, %file%
   {
      Menu, TRAY, Icon, %file%, %number%, 1  ; 1=freeze icon
      If ( Not ErrorLevel )	; if ErrorLevel was NOT raised -> allright
         Goto, OKout
   }

   ; if ErrorLevel or not existing file -> set default icon
   Menu, TRAY, Icon, %h_StarterIcon%, %h_StarterIcon#%, 1 ; 1=freeze icon

   OKout:
      Menu, TRAY, Icon	; show icon
      Menu, TRAY, UseErrorLevel, Off
   Return
}

f_AssignHotkey( hkey, hlabel="" )
{  ; - assign exactly one hotkey to a label
   ; - return TRUE if assignment succeeded
   ;   otherwise FALSE
   ; - remove hotkey if no label is passed
   ;   (return value is not relevant here)
   retVal := FALSE
   ; disable any previosly assigned hotkey
   Hotkey, %hkey%, Off, UseErrorLevel
   ; if no label is specified -> only remove given hotkey
   If IsLabel( hlabel )
   {  ; assign new key
      Hotkey, %hkey%, %hlabel%, UseErrorLevel
      If ( Not ErrorLevel )
         Hotkey, %hkey%, On
            If ( Not ErrorLevel )
               retVal := TRUE
   }
   Return retVal
}

f_ComposeHotkey( chNewKey, chWinKey, chKey, chLabel, chRegSub, chRegKey, chErrorMsg )
{  ; check hotkey, assign and write it to registry
   ; returns new hotkey as string
   ; or "INVALID_KEY" if assignment failed
   ; or "" if hotkey was removed
   Global h_RegSubkey, lng_SetupInvalidHotkey1, lng_SetupInvalidHotkey2

   If ( chNewKey = "" )
   {
      RegDelete, HKCU, %h_RegSubkey%\%chRegSub%, %chRegKey%
      f_AssignHotkey( chKey ) ; remove old hotkey
      chKey =
   }
   Else
   {
      ; do some substitution (for authors convenience)
      StringReplace, chNewKey, chNewKey, ZIRKUMFLEX, SC029 
      StringReplace, chNewKey, chNewKey, AKUT, SC00D 

      ; add WIN modifier if option checked
      If ( chWinKey )
         chNewKey = #%chNewKey%

      ; is there not at least one modifier (WIN, CTRL, ALT, SHIFT) in the new key
      If chNewKey not contains #,^,!,+
         chNewKey = 

      ; trying to assign new hotkey
      success := f_AssignHotkey( chNewKey, chLabel )
      If ( success )
      {
         chKey = %chNewKey%
         RegWrite, REG_SZ, HKCU, %h_RegSubkey%\%chRegSub%, %chRegKey%, %chKey%
      }
      Else
      {
         chKey = INVALID_KEY
         Msgbox, 48, %h_MenuWin%, %lng_SetupInvalidHotkey1%%chErrorMsg%%lng_SetupInvalidHotkey2%
      }
   }
   ; return the new Key (or "") to be used outside this function
   Return chKey
}

f_TrayTip( txt="", secs=5 )
{  ; show a traytip
   Global lng_WindowTitle, m2t_DontBugMe

   If ( m2t_DontBugMe )
      Return

   If ( txt = "" )
   {  ; remove traytip
      TrayTip
      Return
   }
   
   If ( secs >= 1 AND secs <= 20 )
      msecs := secs * 1000
   Else
      msecs = 5000

   TrayTip, %lng_WindowTitle%, %txt%, 20, 17 ; nopopupsound+info
   SetTimer, f_TTRemoveTrayTip, %msecs%
   Return
   
   f_TTRemoveTrayTip:
      SetTimer, f_TTRemoveTrayTip, Off
      TrayTip
      Return
}

f_StringLeft( txt, len=80 )
{  ; get string from left with custom length
   If ( StrLen( txt ) > len )
      StringLeft, txt, txt, %len%
   Return txt
}

;EOF
